#include "EntityFactory.h"

#include "VelocityComponent.h"
#include "TransformableComponent.h"
#include "PlayerComponent.h"
#include "BeamComponent.h"
#include "AngularVelComponent.h"
#include "CollisionComponent.h"
#include "HealthComponent.h"
#include "AnimationComponent.h"
#include "ExpiresComponent.h"
#include "Utility.h"
#include "Constants.h"

std::shared_ptr<artemis::Entity> EntityFactory::rootNode = NULL;
std::shared_ptr<artemis::Entity> EntityFactory::player = NULL;

artemis::Entity& EntityFactory::CreatePlayer(artemis::World& world, const sf::Vector2f& pos)
{
	artemis::Entity & player = world.getEntityManager()->create();
	/*sf::Texture& shipBody = TextureManager::GetTexture("assets/img/ship_body1.png");
	sf::Texture& shipWing = TextureManager::GetTexture("assets/img/ship_wing1.png");
	sf::Texture& shipGun = TextureManager::GetTexture("assets/img/ship_gun1.png");

	sf::RenderTexture& playerTexture = TextureManager::CreateRenderTexture();
	playerTexture.create(shipBody.getSize().y, shipBody.getSize().x + shipWing.getSize().x*2);
	sf::Sprite shipBodySprite(shipBody);
	shipBodySprite.setOrigin(shipBody.getSize().x/2.0f, shipBody.getSize().y/2.0f);
	shipBodySprite.setPosition(playerTexture.getSize().x/2.0f, playerTexture.getSize().y/2.0f);
	shipBodySprite.setRotation(90);
	playerTexture.draw(shipBodySprite);

	sf::Sprite shipWingSprite(shipWing);
	shipWingSprite.setPosition(playerTexture.getSize().x/2.0f + 10.0f, playerTexture.getSize().y/2.0f - 5.0f);
	shipWingSprite.setRotation(90);
	playerTexture.draw(shipWingSprite);
	shipWingSprite.setTextureRect(sf::IntRect(shipWing.getSize().x, 0, -(int)shipWing.getSize().x, shipWing.getSize().y));
	shipWingSprite.setPosition(playerTexture.getSize().x/2.0f + 10.0f, playerTexture.getSize().y/2.0f + 5.0f);
	shipWingSprite.setOrigin(shipWing.getSize().x, 0);
	playerTexture.draw(shipWingSprite);
	playerTexture.display();*/
	
	sf::Texture shipTexture(TextureManager::GetTexture("assets/img/swordfish.png"));
	sf::Sprite shipSprite(shipTexture);
	shipSprite.setOrigin(shipTexture.getSize().x/2.0f, shipTexture.getSize().y/2.0f);
	shipSprite.setPosition(shipTexture.getSize().x/4.0f, shipTexture.getSize().y/4.0f);
	shipSprite.setRotation(90.0f);
	shipSprite.setScale(0.5f, 0.5f);
	sf::RenderTexture& playerTexture = TextureManager::CreateRenderTexture();
	playerTexture.create(shipTexture.getSize().x/2.0f, shipTexture.getSize().y/2.0f);
	playerTexture.draw(shipSprite);

	std::shared_ptr<sf::Sprite> playerSprite(new sf::Sprite(playerTexture.getTexture()));
	playerSprite->setOrigin(playerTexture.getSize().x/2.0f, playerTexture.getSize().y/2.0f);

	player.addComponent(new VelocityComponent(sf::Vector2f()));
	player.addComponent(new HealthComponent(100));
	player.addComponent(new RenderComponent(playerSprite));
	player.addComponent(new TransformableComponent(playerSprite, rootNode->getComponent<TransformableComponent>()));
	player.addComponent(new CollisionComponent(playerSprite->getLocalBounds(), true));
	player.addComponent(new PlayerComponent(sf::Keyboard::W, sf::Keyboard::S,
		sf::Keyboard::A, sf::Keyboard::D, sf::Mouse::Left, sf::Mouse::Right));
	player.refresh();
	player.setEnabled(true);

	if(!world.getTagManager()->isSubscribed(constants::playerTag)) {
		world.getTagManager()->subscribe(constants::playerTag, player);
		EntityFactory::player = std::shared_ptr<artemis::Entity>(&player);
	} else {
		fprintf(stderr, "WARNING: Attempted to tag player twice! Ignored...");
	}

	return player;
}

artemis::Entity& EntityFactory::CreateBeam(artemis::World& world, const sf::Vector2f& pos,
										   const sf::Vector2f& pointPos)
{
	artemis::Entity& beam = world.getEntityManager()->create();
	std::shared_ptr<sf::RectangleShape> rectShape(new sf::RectangleShape());
	//float angle = atan2f(pointPos.y - pos.y, pointPos.x - pos.x)*TO_DEG;
	rectShape->setPosition(48.0f, 0.0f);
	rectShape->setRotation(0.0f);

	beam.addComponent(new RenderComponent(rectShape));
	beam.addComponent(new TransformableComponent(rectShape, player->getComponent<TransformableComponent>()));
	beam.addComponent(new BeamComponent(rectShape, sf::Vector2f(800,10)));
	beam.addComponent(new CollisionComponent(rectShape->getLocalBounds(), false));
	beam.addComponent(new AngularVelComponent(0.0f));
	beam.refresh();

	beam.setEnabled(true);

	world.getGroupManager()->set(constants::beamGroup, beam);

	return beam;
}

artemis::Entity& EntityFactory::CreateBullet(artemis::World& world, const sf::Vector2f& relPos)
{
	artemis::Entity& bullet = world.getEntityManager()->create();
	std::shared_ptr<sf::RectangleShape> rectShape(new sf::RectangleShape(sf::Vector2f(10.0f, 5.0f)));
	sf::Transformable playerTransformable = player->getComponent<TransformableComponent>()->GetGlobalTransformable();
	sf::Vector2f playerVel = player->getComponent<VelocityComponent>()->velocity;

	sf::Transformable relTransformable;
	relTransformable.setPosition(relPos);
	sf::Transformable transformable = compose(playerTransformable, relTransformable);
	copyTransformable(*rectShape, transformable);
	rectShape->setOrigin(5.0f, 2.5f);
	rectShape->setFillColor(sf::Color(255, 0, 0));

	bullet.addComponent(new RenderComponent(rectShape));
	bullet.addComponent(new TransformableComponent(rectShape, rootNode->getComponent<TransformableComponent>()));
	bullet.addComponent(new VelocityComponent(sf::Vector2f(0.0f, 1000.0f) + playerVel));
	bullet.addComponent(new ExpiresComponent(sf::seconds(5.0f)));
	bullet.refresh();

	bullet.setEnabled(true);

	bullet.getComponent<VelocityComponent>()->velocity = sf::Vector2f(0.0f, 1000.0f) + playerVel;

	world.getGroupManager()->set(constants::bulletGroup, bullet);

	return bullet;
}

artemis::Entity& EntityFactory::CreateStaticTest(artemis::World& world, const sf::Vector2f& pos)
{
	artemis::Entity& test = world.getEntityManager()->create();
	sf::Texture& testTexture = TextureManager::GetTexture("assets/img/what.png");
	std::shared_ptr<sf::Sprite> whatSprite(new sf::Sprite(testTexture));
	whatSprite->setPosition(pos);

	test.addComponent(new HealthComponent(100));
	test.addComponent(new RenderComponent(whatSprite));
	test.addComponent(new TransformableComponent(whatSprite, rootNode->getComponent<TransformableComponent>()));
	test.addComponent(new CollisionComponent(whatSprite->getLocalBounds(), true));
	test.refresh();

	test.setEnabled(true);

	world.getGroupManager()->set(constants::enemyGroup, test);

	return test;
}

artemis::Entity& EntityFactory::CreateExplosion(artemis::World& world, const sf::Vector2f& pos)
{
	artemis::Entity& explosion = world.getEntityManager()->create();
	sf::Texture& explosionTexture = TextureManager::GetTexture("assets/img/explosion.png");
	std::shared_ptr<sf::Sprite> explosionSprite(new sf::Sprite(explosionTexture));
	explosionSprite->setPosition(pos);

	explosion.addComponent(new RenderComponent(explosionSprite));
	explosion.addComponent(new TransformableComponent(explosionSprite, rootNode->getComponent<TransformableComponent>()));
	explosion.addComponent(new AnimationComponent(sf::Vector2i(64, 64), "explosion", sf::seconds(0.02f)));
	explosion.addComponent(new ExpiresComponent(sf::seconds(1.0f)));
	explosion.refresh();

	explosion.setEnabled(true);

	AnimManager::AddAnim(*explosion.getComponent<AnimationComponent>(), "explode", 0, 24, false);
	AnimManager::PlayAnim(*explosion.getComponent<AnimationComponent>(), "explode");

	//world.getGroupManager()->set(constants::enemyGroup, test);

	return explosion;
}

artemis::Entity& EntityFactory::CreateStatic(artemis::World& world, const sf::Vector2f& pos, std::string imgLoc) {
	artemis::Entity& spriteEntity = world.getEntityManager()->create();
	sf::Texture& texture = TextureManager::GetTexture(imgLoc);
	std::shared_ptr<sf::Sprite> sprite(new sf::Sprite(texture));
	sprite->setOrigin(texture.getSize().x/2.0f, texture.getSize().y/2.0f);
	sprite->setPosition(pos);

	spriteEntity.addComponent(new RenderComponent(sprite));
	spriteEntity.addComponent(new TransformableComponent(sprite, rootNode->getComponent<TransformableComponent>()));
	spriteEntity.refresh();

	spriteEntity.setEnabled(true);

	return spriteEntity;
}

artemis::Entity& EntityFactory::CreateCamera(artemis::World& world, const sf::Vector2f& pos) {
	artemis::Entity& camera = world.getEntityManager()->create();

	camera.addComponent(new TransformableComponent(rootNode->getComponent<TransformableComponent>()));
	camera.refresh();

	if(!world.getTagManager()->isSubscribed(constants::cameraTag)) {
		world.getTagManager()->subscribe(constants::cameraTag, camera);
	}

	return camera;
}

artemis::Entity& EntityFactory::CreateRootNode(artemis::World& world) {
	artemis::Entity& rootNode = world.getEntityManager()->create();

	rootNode.addComponent(new TransformableComponent(NULL));
	//rootNode.getComponent<TransformableComponent>()->transformable->setRotation(90);
	rootNode.refresh();

	if(!world.getTagManager()->isSubscribed(constants::rootNodeTag)) {
		world.getTagManager()->subscribe(constants::rootNodeTag, rootNode);
		EntityFactory::rootNode = std::shared_ptr<artemis::Entity>(&rootNode);
	} else {
		fprintf(stderr, "WARNING: Attempted to create root node twice! Ignored...");
	}

	return rootNode;
}