
#include "World.h"
#include "Entity.h"
#include "SystemManager.h"

#include <SFML/Graphics.hpp>

#include "MovementSystem.h"
#include "AngularSystem.h"
#include "RenderSystem.h"
#include "RenderBoxSystem.h"
#include "PlayerInputSystem.h"
#include "BeamSystem.h"
#include "Utility.h"
#include "CollisionSystem.h"
#include "DestroySystem.h"
#include "AnimSystem.h"
#include "ExpiresSystem.h"
#include "CameraFollowSystem.h"
#include "SATCollision.h"

#include "EntityFactory.h"


#include "TransformableComponent.h"
#include "Constants.h"

void printTransform(sf::Transform transform) {
	const float* matrix = transform.getMatrix();
	for(int i = 0; i < 4; i++) {
		printf("[ %g, %g, %g, %g]\n", matrix[i*4], matrix[i*4+1], matrix[i*4+2], matrix[i*4+3]);
	}
	printf("\n\n");
}

int main(int argc, char **argv)
{
	/*SATCollision::TestCollision();
	return 0;*/

	const int winWidth = 800;
	const int winHeight = 600;

	initRandom();

	sf::RenderWindow window(sf::VideoMode(winWidth, winHeight), "Test");
	sf::Clock deltaTimer;

	sf::Time frameTime = sf::seconds(1.0f/60.0f);

    artemis::World world;

	EntityFactory::CreateRootNode(world);
	EntityFactory::CreateStatic(world, sf::Vector2f(0.0f, 0.0f), "assets/img/background.png");
	EntityFactory::CreatePlayer(world, sf::Vector2f(0.0f, 0.0f));
	artemis::Entity& camera = EntityFactory::CreateCamera(world, sf::Vector2f(0.0f, 0.0f));

    artemis::SystemManager * sm = world.getSystemManager();
    MovementSystem * movementsys = (MovementSystem*)sm->setSystem(new MovementSystem());
	AngularSystem * angularsys = (AngularSystem*)sm->setSystem(new AngularSystem());
    PlayerInputSystem * inputsys = (PlayerInputSystem*)sm->setSystem(new PlayerInputSystem());
	BeamSystem* beamsys = (BeamSystem*)sm->setSystem(new BeamSystem());
	RenderSystem * rendersys = (RenderSystem*)sm->setSystem(new RenderSystem(window));
	//RenderBoxSystem * renderboxsys = (RenderBoxSystem*)sm->setSystem(new RenderBoxSystem(window));
	CollisionSystem * collisionsys = (CollisionSystem*)sm->setSystem(new CollisionSystem());
	DestroySystem * destroysys = (DestroySystem*)sm->setSystem(new DestroySystem());
	AnimSystem * animatesys = (AnimSystem*)sm->setSystem(new AnimSystem());
	ExpiresSystem * expiresys = (ExpiresSystem*)sm->setSystem(new ExpiresSystem());
	CameraFollowSystem * camerafollowsys = (CameraFollowSystem*)sm->setSystem(new CameraFollowSystem());
    artemis::EntityManager * em = world.getEntityManager();

    sm->initializeAll();

	bool running = true;

    while(running){
		world.loopStart();
        world.setDelta(deltaTimer.getElapsedTime().asSeconds());
		deltaTimer.restart();

		sf::Transform cameraTransform = camera.getComponent<TransformableComponent>()->transformable->getTransform();
		sf::Vector2f transformMousePos = cameraTransform.transformPoint(sf::Mouse::getPosition(window).x, sf::Mouse::getPosition(window).y);
		inputsys->SetMouse(transformMousePos);
		beamsys->SetMouse(transformMousePos);

		sf::Event anEvent;
		while(window.pollEvent(anEvent)) {
			if(anEvent.type == sf::Event::KeyPressed) {
				inputsys->KeyDown(anEvent.key.code);
			} else if(anEvent.type == sf::Event::KeyReleased) {
				inputsys->KeyRelease(anEvent.key.code);
			} else if(anEvent.type == sf::Event::MouseButtonPressed) {
				inputsys->MousePress(anEvent.mouseButton.button);
			} else if(anEvent.type == sf::Event::MouseButtonReleased) {
				inputsys->MouseRelease(anEvent.mouseButton.button);
			} else if(anEvent.type == sf::Event::Closed) {
				running = false;
			}
		}


		expiresys->process();
		inputsys->process();
		
        movementsys->process();
		camerafollowsys->process();

		angularsys->process();
		beamsys->process();
		collisionsys->process();
		destroysys->process();

		animatesys->process();

		window.clear();
		rendersys->process();
		//renderboxsys->process();
		window.display();

		sf::Time timePassed = deltaTimer.getElapsedTime();
		if(timePassed < frameTime) {
			sf::sleep(frameTime - timePassed);
		}

    }

    return 0;
}
