
#include "MovementSystem.h"

#include "TransformableComponent.h"
#include "VelocityComponent.h"

#include "Utility.h"

MovementSystem::MovementSystem() {
	addComponentType<VelocityComponent>();
	addComponentType<TransformableComponent>();
};

void MovementSystem::initialize() {
	velocityMapper.init(*world);
	transformableMapper.init(*world);
};

void MovementSystem::processEntity(artemis::Entity &e) {
	sf::Vector2f entityPos = transformableMapper.get(e)->transformable->getPosition();
	sf::Vector2f entityVel = velocityMapper.get(e)->velocity;
	sf::Vector2f entityAccel = velocityMapper.get(e)->accel;
	sf::Vector2f entityVelMax = velocityMapper.get(e)->velocityMax;

	if(entityAccel.x == 0) {
		entityAccel.x = -sign(entityVel.x) * std::min(abs(entityVel.x), 200.0f);
	}
	if(abs(entityAccel.x) < 1.0f) {
		entityVel.x = 0.0f;
	}
	if(entityAccel.y == 0) {
		entityAccel.y = -sign(entityVel.y) * std::min(abs(entityVel.y), 400.0f);
	}
	if(abs(entityAccel.y) < 1.0f) {
		entityVel.y = 0.0f;
	}

	sf::Vector2f newEntityVel = entityVel + entityAccel * world->getDelta();
	if(std::abs(newEntityVel.x) > std::abs(entityVelMax.x)) {
		entityVel.x = sign(newEntityVel.x)*entityVelMax.x;
	} else {
		entityVel.x = newEntityVel.x;
	}
	if(std::abs(newEntityVel.y) > std::abs(entityVelMax.y)) {
		entityVel.y = sign(newEntityVel.y)*entityVelMax.y;
	} else {
		entityVel.y = newEntityVel.y;
	}

	transformableMapper.get(e)->transformable->rotate(entityVel.x * world->getDelta());
	float rotation = transformableMapper.get(e)->transformable->getRotation();
	transformableMapper.get(e)->transformable->setPosition(entityPos
		+ sf::Vector2f(entityVel.y * cos(rotation*TO_RAD) * world->getDelta(), entityVel.y * sin(rotation*TO_RAD) * world->getDelta()));

	velocityMapper.get(e)->velocity = entityVel;
};