
#ifndef __PLAYERINPUTSYSTEM_H
#define __PLAYERINPUTSYSTEM_H

#include "EntityProcessingSystem.h"
#include "ComponentMapper.h"

class VelocityComponent;
class TransformableComponent;
class PlayerComponent;

#include "SFML/Window.hpp"

#include <cmath>
#include <vector>

class PlayerInputSystem : public artemis::EntityProcessingSystem {
private:
    artemis::ComponentMapper<VelocityComponent> velocityMapper;
	artemis::ComponentMapper<TransformableComponent> TransformableMapper;
	artemis::ComponentMapper<PlayerComponent> playerMapper;

	sf::Vector2f mousePt;
	std::vector<bool> keyDown;
	std::vector<bool> mouseDown;
public:
    PlayerInputSystem();

    virtual void initialize();

    virtual void processEntity(artemis::Entity &e);

	void SetMouse(sf::Vector2f mouse);

	void KeyDown(sf::Keyboard::Key key);

	void KeyRelease(sf::Keyboard::Key key);

	void MousePress(sf::Mouse::Button button);

	void MouseRelease(sf::Mouse::Button button);

	const static float playerMoveSpeed;
	const static float playerSlowMoveSpeed;
	const static float playerTurnSpeed;
	const static float playerSlowTurnSpeed;
	const static sf::Time powerDownTime;
	const static sf::Vector2f maxPlayerMoveSpeed;
	const static sf::Vector2f maxPlayerSlowMoveSpeed;
};

#endif