
#include "RenderSystem.h"
#include "RenderComponent.h"
#include "TransformableComponent.h"

#include "Entity.h"
#include "Constants.h"

RenderSystem::RenderSystem(sf::RenderWindow& window) :
	window(window)
{
	addComponentType<RenderComponent>();
	addComponentType<TransformableComponent>();
};

void RenderSystem::initialize()
{
	RenderMapper.init(*world);
	TransformableMapper.init(*world);
};

void RenderSystem::processEntities(artemis::ImmutableBag<artemis::Entity*> & entities) {
	sf::Transformable* cameraTransformable = world->getTagManager()->getEntity(constants::cameraTag)
		.getComponent<TransformableComponent>()->transformable.get();
	sf::Transform cameraTransform = world->getTagManager()->getEntity(constants::cameraTag)
		.getComponent<TransformableComponent>()->transformable->getTransform().getInverse();
	for(int i = 0; i < entities.getCount(); i++) {
		TransformableComponent* transformableComponent = entities.get(i)->getComponent<TransformableComponent>();
		sf::Transformable transformable = transformableComponent->GetParentTransformable();

		sf::Transform combinedTransform = cameraTransform;
		combinedTransform.combine(transformable.getTransform());

		window.draw(*entities.get(i)->getComponent<RenderComponent>()->drawable, sf::RenderStates(combinedTransform));
	}
}

bool RenderSystem::checkProcessing() {
	return true;
}
