
#ifndef __TRANSFORMABLECOMPONENT_H
#define __TRANSFORMABLECOMPONENT_H

#include "Component.h"
#include "Utility.h"

#include "SFML/Graphics.hpp"

#include <memory>
#include <list>

class TransformableComponent : public artemis::Component
{
public:
	TransformableComponent(const TransformableComponent* parent) :
		parent(parent)
	{
		transformable = std::shared_ptr<sf::Transformable>(new sf::Transformable());
		transformable->setPosition(0, 0);
		transformable->setRotation(0);
		transformable->setScale(1, 1);
	}

    TransformableComponent(std::shared_ptr<sf::Transformable> transformable, const TransformableComponent* parent) :
		transformable(transformable) ,
		parent(parent)
	{ }

	sf::Transformable GetParentTransformable() const {
		if(parent != NULL) {
			return parent->GetGlobalTransformable();
		} else {
			return sf::Transformable();
		}
	}

	sf::Transformable GetGlobalTransformable() const {
		if(parent != NULL) {
			sf::Transformable parentTransformable(parent->GetGlobalTransformable());
			return compose(parentTransformable, *transformable);
		} else {
			return *transformable;
		}
	}

	//Transformable, in global coordinates (where it actually should be drawn)
	std::shared_ptr<sf::Transformable> transformable;

	//Parent transform, whose transformations are composed
	// DON'T use a pointer from the std library; if this is the only reference
	// to the transformable, it will be deleted!
	const TransformableComponent* parent;
};

#endif