
#include "Utility.h"

std::mt19937 randgen;

float magnitude(const sf::Vector2f& vec)
{
	return sqrt(vec.x*vec.x+vec.y*vec.y);
}

float shiftAngle(float angle)
{
	while(angle < -PI || angle > PI) {
		if(angle < -PI) { 
			angle += 2*PI;
		} else if(angle > PI) {
			angle -= 2*PI;
		}
	}

	return angle;
}

void initRandom()
{
	randgen.seed((unsigned long)time(NULL));
}

float getRandom()
{
	return (float)randgen()/(float)randgen.max();
}

float getRandom(float begin, float end)
{
	return getRandom()*(end-begin)+begin;
}

bool IsValidProperty(const std::string& prop)
{
	return !prop.empty() && prop.compare("No such property!") != 0;
}

int sign(float val)
{
	if(val < 0)
		return -1;
	if(val > 0)
		return 1;
	return 0;
}

float dot(const sf::Vector2f& a, const sf::Vector2f& b)
{
	return a.x*b.x + a.y*b.y;
}

sf::Vector2f innerProduct(const sf::Vector2f& a, const sf::Vector2f& b)
{
	return sf::Vector2f(a.x*b.x, a.y*b.y);
}

sf::Transformable compose(const sf::Transformable& parent, const sf::Transformable& child) {
	sf::Transformable t;

	float thetaRad = parent.getRotation() * TO_RAD;
	sf::Vector2f childPos = child.getPosition();
	sf::Vector2f transPos;
	transPos.x = cos(thetaRad) * childPos.x - sin(thetaRad) * childPos.y;
	transPos.y = sin(thetaRad) * childPos.x + cos(thetaRad) * childPos.y;

	t.setPosition(parent.getPosition() + transPos);
	t.setRotation(parent.getRotation() + child.getRotation());
	t.setScale(innerProduct(parent.getScale(), child.getScale()));
	//t.setOrigin(child.getOrigin());
	return t;
}

void copyTransformable(sf::Transformable& to, const sf::Transformable& from) {
	to.setPosition(from.getPosition());
	to.setRotation(from.getRotation());
	to.setScale(from.getScale());
}